'''
    Convert to django
    for covert output bs_get_all_text : res.html and src.html to django template
    django template: 
        src.html ->
            [code]-base.html
            [code]-base-index.html
            [code]-index.html
        code = hexadesimal uniq

        res.html ->
            [code]-base.html
            [code]-base-index.html
            [code]-index.html
'''

import re
# import json
from bs4 import BeautifulSoup
# import copy
from pathlib import Path

def replace_title(input_file, template_name):    
    # input_name = input_file.split(".")[0]

    with open(input_file, 'r', encoding='utf-8') as file:
        html_content = file.read()
    
    soup = BeautifulSoup(html_content, 'html.parser')
    text_elements = soup.find('title')
    
    # add include id base.html
    tmp = "{{ title }}"
    # m_include = BeautifulSoup(tmp, "html.parser")
    text_elements.string.replace_with(tmp)

    # Add extra head dan extra body di bagian akhir tag penutup
    tmp = "{% block extra_head %} {% endblock %}"
    m_include = BeautifulSoup(tmp, "html.parser")
    soup.head.append(m_include)

    tmp = "{% block extra_body %} {% endblock %}"
    m_include = BeautifulSoup(tmp, "html.parser")
    soup.body.append(m_include)

    with open(template_name + '/' + 'base.html', 'w', encoding='utf-8') as file:        
        file.write(soup.prettify())

    # remove all link element from source html
    print("Add Extra head and body Done...")
    print("Replace title Done...")
    return soup

    # with open('extracted_texts.json', 'w', encoding='utf-8') as json_file:
    #     json.dump(texts, json_file, ensure_ascii=False, indent=4)
    
    # with open('snippets/style.html', 'r', encoding='utf-8') as file:
    #     for i in locals
    #     file.write(soup.prettify())
    # print("Extract Link Done...")

# Gak bisa cara ini, harus cek lagi has_attr di caller function
# def add_static_to(element, folder_name, attr):
#     if element.has_attr(attr):
#         if not("https://" in element[attr]):            
#             return "{% static '" + folder_name + "/" + element[attr] + "' %}"
#     return element

def extract_tag_name(soup, folder_name, tag_name, m_index=0):        
    # find all tag
    tag_elements = soup.find_all(tag_name)
    
    # add static to target html except for meta
    link_element = ""
    if tag_name!="meta":
        link_element = "{% load static %}"

    # proses all finding tag
    m_parent_tag = None
    for i in range(len(tag_elements)):
        # print('parent name', tag_elements[i].parent.name)
        if not m_parent_tag:
            m_parent_tag = tag_elements[i].parent

        if tag_elements[i].has_attr('href'):
            if not("https://" in tag_elements[i]["href"]):            
                tag_elements[i]["href"] = "{% static '" + folder_name + "/" + \
                                            tag_elements[i]["href"] + "' %}"
                
        if tag_elements[i].has_attr('src'):
            if not("https://" in tag_elements[i]["src"]):            
                tag_elements[i]["src"] = "{% static '" + folder_name + "/" + \
                                            tag_elements[i]["src"] + "' %}"
                
        link_element += tag_elements[i].prettify()
        # hapus element link dari soup
        tag_elements[i].decompose()
        
    # write to file tag link or meta (tag_name)
    m_link = BeautifulSoup(link_element, 'html.parser')    
    with open(folder_name + '/snippets/'+ tag_name +'.html', 'w', encoding='utf-8') as file:        
        file.write(m_link.prettify())

    # add include to base.html
    if tag_name != 'main':
        tmp = "{% include '"+ folder_name +"/snippets/"+ tag_name +".html' %}"
    else:
        tmp = "{% block "+ tag_name +" %}"+"{% endblock %}"

    m_include = BeautifulSoup(tmp, "html.parser")
    if tag_name=='script':
        # print('YOU MUST FIND extra_body HERE')
        tmp = soup.find(string=re.compile('extra_body'))
        if tmp:
            tmp.insert_before(m_include)
        else:
            print('extra_body not found!')
    else:
        m_parent_tag.insert(m_index, m_include)
    
    # soup.head.append(m_include)
    with open(folder_name + '/base.html', 'w', encoding='utf-8') as file:        
        file.write(soup.prettify())

    # remove all link element from source html
    print(f"Extract {tag_name} Done...")
    return soup

    # with open('extracted_texts.json', 'w', encoding='utf-8') as json_file:
    #     json.dump(texts, json_file, ensure_ascii=False, indent=4)
    
    # with open('snippets/style.html', 'r', encoding='utf-8') as file:
    #     for i in locals
    #     file.write(soup.prettify())
    # print("Extract Link Done...")

# def extract_link(folder_name, soup):    
#     # with open(input_file, 'r', encoding='utf-8') as file:
#     #     html_content = file.read()
    
#     # soup = BeautifulSoup(html_content, 'html.parser')
#     text_elements = soup.find_all('link')
    
#     # texts = [element.strip() for element in text_elements if element.strip()]
#     link_element = "{% load static %}"
#     for i in range(len(text_elements)):
#         if not("https://" in text_elements[i]["href"]):
#             text_elements[i]["href"] = "{% static '" + folder_name + "/" + \
#                                         text_elements[i]["href"] + "' %}"
#         link_element += text_elements[i].prettify()
#         # hapus element link dari soup
#         text_elements[i].decompose()
        
#     m_link = BeautifulSoup(link_element, 'html.parser')    

#     with open('snippets/style.html', 'w', encoding='utf-8') as file:        
#         file.write(m_link.prettify())

#     # add include id base.html
#     tmp = "{% include '"+ folder_name +"/snippets/style.html' %}"
#     m_include = BeautifulSoup(tmp, "html.parser")
#     soup.head.append(m_include)

#     with open('base.html', 'w', encoding='utf-8') as file:        
#         file.write(soup.prettify())

#     # remove all link element from source html
#     print("Extract Link Done...")
#     return soup

#     # with open('extracted_texts.json', 'w', encoding='utf-8') as json_file:
#     #     json.dump(texts, json_file, ensure_ascii=False, indent=4)
    
#     # with open('snippets/style.html', 'r', encoding='utf-8') as file:
#     #     for i in locals
#     #     file.write(soup.prettify())
#     # print("Extract Link Done...")

# def extract_meta(folder_name, soup):    
#     # with open(input_file, 'r', encoding='utf-8') as file:
#     #     html_content = file.read()
    
#     # soup = BeautifulSoup(html_content, 'html.parser')
#     text_elements = soup.find_all('meta')
    
#     # texts = [element.strip() for element in text_elements if element.strip()]
#     link_element = ""
#     for i in range(len(text_elements)):        
#         link_element += text_elements[i].prettify()
#         # hapus element link dari soup
#         text_elements[i].decompose()
        
#     m_link = BeautifulSoup(link_element, 'html.parser')    

#     with open('snippets/meta.html', 'w', encoding='utf-8') as file:        
#         file.write(m_link.prettify())


#     # add include id base.html
#     tmp = "{% include '"+ folder_name +"/snippets/meta.html' %}"
#     m_include = BeautifulSoup(tmp, "html.parser")
#     soup.head.append(m_include)

#     with open('base.html', 'w', encoding='utf-8') as file:        
#         file.write(soup.prettify())

#     print("Extract Meta Done...")
#     return soup
#     # remove all link element from source html
#     # with open('extracted_texts.json', 'w', encoding='utf-8') as json_file:
#     #     json.dump(texts, json_file, ensure_ascii=False, indent=4)
    
#     # with open('snippets/style.html', 'r', encoding='utf-8') as file:
#     #     for i in locals
#     #     file.write(soup.prettify())

#     # print("Extract Meta Done...")

# # def extract_body(folder_name, soup):
#     body_tag = soup.body

#     first_tag = body_tag.find_next()
#     # if tag = header, or footer
#     print('first_tag.name', first_tag.name)

    
#     # Find all elements after body
#     # elements_after_body = []
#     # current = body_tag.next_sibling

#     # while current:
#     #     if current.name is not None: # This is a tag
#     #         elements_after_body.append(current)
#     #     current = current.next_sibling

#     # # Display results
#     # print(f"Found {len(elements_after_body)} tags after the body:")
#     # for i, element in enumerate(elements_after_body, 1):
#     #     print(f"{i}. {element.name}: {element}")

#     # all_tags_after_body = body_tag.find_all_next(limit=5)
#     # # print('ALL NEXT', all_tags_after_body)
#     # for i in all_tags_after_body:
#     #     print('--->', i)

#     return

#     # Find the first tag after <body>
#     first_tag_after_body = body_tag.find_next()
#     print('first_tag_after_body', first_tag_after_body)

#     first_tag_after_body = first_tag_after_body.find_next()
#     print('second_tag_after_body', first_tag_after_body)

def travel_tag_head(soup):
    '''
        Travel from first root tag to the end 
    '''
    element = soup.head.find_next()
    # print('element', element)
    # idx = 0
    m_arr = []
    m_exclude = ['title']
    while element:        
        # idx += 1
        # print(f'{idx} - {element.name}')
        tmp = element.name
        if tmp.lower() not in m_exclude:                                
            if tmp not in m_arr:
                m_arr.append(tmp)            

        element = element.find_next_sibling()
    return m_arr
    # print('m_arr', m_arr)
        
def travel_tag_body(soup):
    '''
        Travel from first root tag to the end 
    '''
    element = soup.body.find_next()
    # print('element', element)
    # idx = 0
    m_arr = []
    # 'main',  tidak digunakan karena tetap menggunakan algoritma seperti tag header dan footer, dan di proses ulang file dan inlcude di proses berikutnya
    m_exclude = ['div', 'nav', 'ul', 'p', 'a'] # main di gunakan untuk sub file (base-index.html)
    while element:        
        # idx += 1
        # print(f'{idx} - {element.name}')
        tmp = element.name
        if tmp.lower() not in m_exclude:                                
            if tmp not in m_arr:
                m_arr.append(tmp)            

        element = element.find_next_sibling()
    return m_arr
    # print('m_arr', m_arr)

# def travel_tag_body_main(soup):
#     '''
#         Travel from first root tag to the end 
#     '''
#     element = soup.body.main.find_next()    # tag main harus ada, jika container, ubah menjadi main (di index.html) --> proses normalisasi
#     # print('element', element.name)
#     # idx = 0
#     m_arr = []
#     m_exclude = ['div', 'nav', 'ul', 'p', 'a']
#     while element:        
#         # idx += 1
#         # print(f'{idx} - {element.name}')
#         tmp = element.name
#         if tmp.lower() not in m_exclude:                                
#             if tmp not in m_arr:
#                 m_arr.append(tmp)            

#         element = element.find_next_sibling()
#     # print('m_arr', m_arr)
#     return m_arr

def scrape_head_body(input_file, template_name):
    soup = replace_title(input_file, template_name)

    m_tag_head = travel_tag_head(soup)
    for i in range(len(m_tag_head)):
        soup = extract_tag_name(soup, template_name, m_tag_head[i], m_index=i)
    
    # soup = extract_tag_name(soup, template_name, 'meta', m_index=1)
    # soup = extract_body(template_name, soup)

    m_tag_body = travel_tag_body(soup)
    for i in range(len(m_tag_body)):
        soup = extract_tag_name(soup, template_name, m_tag_body[i], m_index=i)


    # soup = extract_tag_name(soup, template_name, 'header')
    # soup = extract_tag_name(soup, template_name, 'main', m_index=1)
    # soup = extract_tag_name(soup, template_name, 'footer', m_index=2)
    # # append (insert to the last position)
    # soup = extract_tag_name(soup, template_name, 'script', m_index=3)

    # travel_tag_head(soup)
    # travel_tag_body(soup)
    # m_main = travel_tag_body_main(soup)
    # for i in range(len(m_main)):
        # soup = extract_tag_name(soup, template_name, m_main[i], m_index=i)
    
    print("Scrape head and body Done...")

def create_index_and_base(input_file, template_name, m_section_list):
    '''
        Create base-index.html
        Create index.html
        base-index order by user data in (m_section_list)
        Sumber data data snippets/main.html (biarkan untuk proses reorder lagi oleh user)
    '''
    # open main.html
    with open(template_name + '/' + input_file, 'r', encoding='utf-8') as file:
        html_content = file.read()
    
    soup = BeautifulSoup(html_content, 'html.parser')
    main_tag= soup.main.name
    m_main = soup.main

    # print ('m_main', m_main.name)

    # Create header base-index
    tmp = """{% extends '""" + template_name + """/base.html' %}
        {% load static %}
        {% block """+ main_tag +""" %}
        """
    soup_base_index = BeautifulSoup(tmp, 'html.parser')

    tmp = """{% extends '""" + template_name + """/base-index.html' %}
        {% load static %}        
        """
    soup_index = BeautifulSoup(tmp, 'html.parser')


    # proses untuk kedua soup, index, dan base-index
    for i in m_section_list:
        elem = m_main.find('section',id=i)
        if elem:
            ret = elem.extract()
            tmp = "{% block "+ i +" %}"
            soup_index.append(BeautifulSoup(tmp, 'html.parser'))
            soup_index.append(ret)
            tmp = "{% endblock %}"
            soup_index.append(BeautifulSoup(tmp, 'html.parser'))
                    
        
    # proses urutan section di base-index.html
    for i in range(len(m_section_list)):
        tmp = "{% block "+ m_section_list[i] +" %} {% endblock %}"
        m_main.insert(i, BeautifulSoup(tmp, 'html.parser'))
    soup_base_index.append(m_main)
    

    # create footer base index
    tmp = '{% endblock %}'
    soup_base_index.append(BeautifulSoup(tmp, 'html.parser'))                

    with open(template_name + '/' + 'base-index.html', 'w', encoding='utf-8') as file:        
        file.write(soup_base_index.prettify())

    with open(template_name + '/' + 'index-modify.html', 'w', encoding='utf-8') as file:        
        file.write(soup_index.prettify())

# def create_index(soup, soup_index, template_name, m_section_name, is_first_enter=False):    
#     # soup ini sudah di dalamnya main
#     # m_main = soup.main
#     # print ('m_main', m_main.name)
#     # {% extends "ilanding/base-index.html" %}
#     # {% load static %}

#     tmp = ""    
#     if is_first_enter:
#         tmp = """{% extends '""" + template_name + """/base-index.html' %}
#                 {% load static %}
#                 """
        
#     tmp +=  """
#             {% block """+ m_section_name +""" %}
#             """
    
#     soup_index.append(BeautifulSoup(tmp, 'html.parser'))

#     element = soup.find('section')
#     # print('element', element)
#     # if element:
#     soup_index.append(copy.deepcopy(element))
    
#     tmp = '{% endblock %}'
#     soup_index.append(BeautifulSoup(tmp, 'html.parser'))    
            
#     # hapus section ini dari base-index.html
#     element.decompose()    
#     # print('element', element.name)

def update_img_src(input_file, template_name):
    # open main.html
    with open(template_name + '/' + input_file, 'r', encoding='utf-8') as file:
        html_content = file.read()
    
    soup = BeautifulSoup(html_content, 'html.parser')
    elem = soup.find_all('img')
    # print('lelem',elem)
    for i in elem:       
        if i.has_attr('src'):
            # print('src', i, i['src'])
            i["src"] = "{% static '" + template_name + "/" + i["src"] + "' %}"

    with open(template_name + '/' + input_file, 'w', encoding='utf-8') as file:
        # html_content = file.read()
        file.write(soup.prettify())
    
def get_all_section_id(input_file, folder_name):
    '''
        Untuk proses shuffle urutal section
        re create disini
    '''
    with open(folder_name + '/' + input_file, 'r', encoding='utf-8') as file:
        html_content = file.read()
    
    soup = BeautifulSoup(html_content, 'html.parser')
    # print('name',soup.main.name)
    m_section = soup.main.find_next()
    # print('m_section', m_section.name)
    m_arr = []
    while m_section:
        if m_section.has_attr('id'):
            # print(m_section.name, m_section['id'])
            m_arr.append(m_section['id'])
        m_section = m_section.find_next_sibling()
    return m_arr

def scrape_all(template_name, input_file):
    # template_name = "ilanding"
    # input_file = 'res.html'  

    input_name = input_file.split(".")[0]
    template_name += '-' + input_name

    folder_path = Path(template_name)
    folder_path.mkdir(parents=True, exist_ok=True)

    folder_path = Path(template_name + "/snippets")
    folder_path.mkdir(parents=True, exist_ok=True)

    scrape_head_body(input_file, template_name)

    # -------------------------------------------
    # lanjut ke proses berikutnya:
    # proses snippets/main.html
    # ubah menjadi base-index.html dan index.html
    # -------------------------------------------
    input_file = 'snippets/main.html'      
    m_section_list = get_all_section_id(input_file, template_name) # <--
    # for i in range(len(m_section_list)):
    #     print('section', i, m_section_list[i])
    # section list dapat di hapus dan di reorder sesuai kebutuhan

    update_img_src(input_file, template_name)

    # # ---------------------------------------------------
    # # Proses m_section_list yg sudah di proses oleh user:
    create_index_and_base(input_file, template_name, m_section_list) # belum ada output ke file, masih di variabel soup


    # # soup_index = copy.deepcopy(soup)
    # soup_index = BeautifulSoup("", 'html.parser')

    # for i in range(len(m_section_list)):        
    #     create_index(soup, soup_index, template_name, m_section_list[i], i==0)
    #     # if i==2:
    #     #     break

    # with open('base-index.html', 'w', encoding='utf-8') as file:        
    #     file.write(soup.prettify())

    # with open('dynamic-index.html', 'w', encoding='utf-8') as file:        
    #     file.write(soup_index.prettify())

    print("All Done...")


import argparse
from bs_get_all_text import scrape_text
from convert_to_django import scrape_all

def main():
    print("Hello, let's convert to django template!")
    parser = argparse.ArgumentParser(description="A simple CLI tool.")
    parser.add_argument("--templatename", help="Template Name (like ilanding)", required=True)
    parser.add_argument("--filepath", help="File path (usually index.html)", required=True)
    args = parser.parse_args()
    # print(f"File Path is, {args.filepath}!")
    scrape_text(args.filepath)
    scrape_all(args.templatename, 'res.html')
    scrape_all(args.templatename, 'src.html')

if __name__ == "__main__":
    main()



"""
Provides authbox_django_template version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update authbox_django_template` to change this file.

from incremental import Version

__version__ = Version("authbox_django_template", 1, 0, 0)
__all__ = ["__version__"]



'''
    2025 Agustus 30
    authbox.web.id
'''
import re
import copy
import json
from bs4 import BeautifulSoup, Comment


def get_root_parent(target, contain_tag_name):
    '''
        # Traverse to the root parent
        # cek apakah mengandung tag tertentu misalnya NAV (jika iya return true)
    '''
    root_parent = target
    while root_parent.parent is not None:
        root_parent = root_parent.parent
        if root_parent.name == contain_tag_name:
            return True 
    return False

def get_root_parent_id(target, contain_tag_name):
    '''
        # Traverse to the root parent
        # cek apakah mengandung id tertentu misalnya id="hero-1" (jika iya return true)
    '''
    root_parent = target
    while root_parent.parent is not None:
        root_parent = root_parent.parent
        if root_parent.name == contain_tag_name:
            if root_parent.has_attr('id'):
                mid = root_parent.get('id')    
                return mid 
    return None 

def get_root_parent_id_name(target, contain_tag_name, contain_id_name):
    '''
        # Traverse to the root parent
        # cek apakah mengandung id tertentu misalnya id="hero-1" (jika iya return true)
    '''
    root_parent = target
    while root_parent.parent is not None:
        root_parent = root_parent.parent
        if root_parent.name == contain_tag_name:
            if root_parent.has_attr('id'):
                mid = root_parent.get('id')   
                if mid == contain_id_name:
                    return True                
    return False

def clean_text(text_to_find, mchar):
    '''
        Clean text by removing specific characters
        Karakter + atau ? atau /g akan di anggap perintah khsusus oleh re.compile
    '''
    if text_to_find[0] == mchar:
        return text_to_find.replace(mchar,'')
    return text_to_find

def get_replacement_soup(text_to_find, m_code, m_type):
    tmp = f"""
            <span class="hoverable hoverable-mark" id='{m_code}'
                data-tooltip='Link ini hanya tampil di mode edit. Silahkan klik untuk mengedit data!'
                data-modal-title='{text_to_find}'
                data-modal-code='{m_code}'
                data-modal-type='{m_type}'>
                {text_to_find}
            </span> 
        """
    # tmp =   """
    #         <span class="hoverable hoverable-mark" id='""" + m_code + """'
    #             data-tooltip="Link ini hanya tampil di mode edit. Silahkan klik untuk mengedit data!" 
    #             data-modal-title='""" + text_to_find + """'
    #             data-modal-code='""" + m_code + """'
    #             data-modal-type='""" + m_type + """'
    #             >
    #         """ + text_to_find + """
    #             </span> 
    #         """            
    return BeautifulSoup(tmp, 'html.parser')    

def get_replacement_soup_copy(text_to_find, m_code, m_type):
    '''
        Untuk source.html yang tidak ada hoverable        
    '''
    tmp = f"""
            <span class="hoverable-mark" id='{m_code}'                                
                data-modal-type='{m_type}'>
                {text_to_find}
            </span> 
        """
    # tmp =   """
    #         <span class="hoverable-mark" id='""" + m_code + """'                
    #             data-modal-type='""" + m_type + """'
    #             >
    #         """ + text_to_find + """
    #             </span> 
            # """            
    return BeautifulSoup(tmp, 'html.parser')   

def add_hover_click(soup, text_to_find, m_section_array, is_copy=False, manifest=None):
    '''
        add hover-text to all kind who user can change
    '''
    # Error jika ada karakter +
    text_to_find = clean_text(text_to_find, '+')
    # print('text_to_find', text_to_find )

    # find all text    
    text_element = soup.body.find_all(string=re.compile(text_to_find))    
    
    for i in range(len(text_element)):
        # cannot continue if text = None (Fix index.html first)
        if not text_element[i]:
            print('Ubah data index.html jika ini masih muncul!!! (Hilangkan karakter ? + dll)')
            print('Text to find:', text_to_find)
            print('Found text_element:', text_element[i])
            print('Fix this first')
            return 

        # add exception
        m_continue = True
        if get_root_parent(text_element[i], 'nav'):            
            m_continue = False 

        if m_continue and get_root_parent(text_element[i], 'ul'):            
            m_continue = False 

        if m_continue and get_root_parent(text_element[i], 'form'):            
            m_continue = False 

        if m_continue and get_root_parent_id_name(text_element[i], 'div', 'myModal'):
            # print('ENTER')
            m_continue = False

        # get section id
        m_section = {}
        if m_continue:
            # get section name
            # reset code begin from 1 if new section name detected
            # if section name exists code continue from last index
            tmp = get_root_parent_id(text_element[i], 'section')            
            if not tmp:                
                tmp = 'main' # default section name (main)                                                   

            mfound = False
            for j in range(len(m_section_array)):
                if m_section_array[j]['name'].strip() == tmp.strip():                        
                    m_section_array[j]['code'] += 1
                    m_section = m_section_array[j]
                    mfound = True
                    break
                
            if not mfound:
                m_section = {'name': tmp, 'code': 1}                        
                m_section_array.append(m_section) 

        # agar tidak 2 kali penulisan hoverable ini        
        if text_element[i].parent.has_attr('class'):
            mclass = text_element[i].parent.get('class')    
            for j in mclass:                
                # class hoverable-mark tetap ada di source.html dan result.html
                # sebagai penanda ini ada hoverable sudah ada dan tidak perlu di create ulang (untuk menghindari perulangan)
                if (j=='hoverable-mark') or (j=='close'):
                    m_continue = False
                    break                
            
        if m_continue:
            m_code = f"{len(m_section_array)}-{m_section['name']}-{m_section['code']}"     
            # print('m_code', m_code)
            if manifest is not None:
                manifest.append(m_code)
            m_type = 'text'               

            if not is_copy:
                replacement_soup = get_replacement_soup(text_to_find, m_code, m_type)            
                # text_element[i].string.replace_with(replacement_soup)  
            else: # print('text_element[i]', text_element[i].parent)
                replacement_soup = get_replacement_soup_copy(text_to_find, m_code, m_type)            
            text_element[i].string.replace_with(replacement_soup)  
                
                # text_element[i]['id'] = m_code

            # for source.html
            # text_element_copy = soup_copy.body.find_all(string=re.compile(text_to_find))          
            # for j in range(len(text_element_copy)):
            #     if text_element_copy[j]:
            #         text_element_copy[j].string.replace_with(text_to_find)
            # text_element_copy = soup.body.find_all(string=re.compile(text_to_find))          
            
def replace_icon(soup, class_name, len_section_array, is_copy=False, manifest=None):
    '''
        Replace all icon with hoverable
        class_name = 'bi' for bootstrap icon
    '''
    bi_icons = soup.find_all('i', class_=class_name)    
    idx = 0
    m_type = 'icon'

    # Print results
    for icon in bi_icons:
        if icon:            
            m_continue = True
            if get_root_parent(icon, 'nav'):                
                m_continue = False 

            if m_continue and get_root_parent(icon, 'ul'):                
                m_continue = False 

            if m_continue and get_root_parent(icon, 'form'):            
                m_continue = False 

            if m_continue:
                idx += 1
                m_code = f'{len_section_array}-bi-icon-{idx}'    
                if manifest is not None:
                    manifest.append(m_code)            

                icon_name = ""
                for j in icon.get('class', []):                    
                    if j.startswith('bi-'):
                        icon_name = j                        
                        break
                
                if icon_name:                    
                    if not is_copy:
                        replacement_soup = get_replacement_soup(icon, m_code, m_type)                    
                    else:
                        replacement_soup = get_replacement_soup_copy(icon, m_code, m_type)                    
                    icon.replace_with(replacement_soup)

def replace_img(soup, len_section_array, is_copy=False, manifest=None):
    '''
        Replace all img with hoverable
        # image tag bisa langsung tanpa pengecekan nav, ul, dan form
    '''
    imgs = soup.find_all('img')    
    idx = 0
    m_type = 'image'
    
    for img in imgs:        
        if img:
            idx += 1
            m_code = f'{len_section_array}-image-{idx}'  
            if manifest is not None:
                manifest.append(m_code)

            if not is_copy:
                replacement_soup = get_replacement_soup(img, m_code, m_type)            
            else:
                replacement_soup = get_replacement_soup_copy(img, m_code, m_type)
            img.replace_with(replacement_soup)

def replace_ul(soup, len_section_array, is_copy=False, manifest=None):
    items = soup.find_all('ul')    
    idx = 0
    m_type = 'unordered-list'

    for item in items:        
        if item:            
            m_continue = True
            if get_root_parent(item, 'nav'):                
                m_continue = False 
            
            if m_continue and get_root_parent(item, 'form'):            
                m_continue = False 

            if m_continue:
                idx += 1
                m_code = f'{len_section_array}-unordered-list-{idx}'
                if manifest is not None:
                    manifest.append(m_code)
                    
                # data-modal-title='{item.get_text(separator=', ', strip=True)}'                
                # print('---', tmp)
                if is_copy:
                    replacement_soup = get_replacement_soup_copy(item, m_code, m_type)                
                else:
                    # data-modal-title='{item.get_text(separator=', ', strip=True)}'
                    replacement_soup = get_replacement_soup(item, m_code, m_type)                
                item.replace_with(replacement_soup)

def replace_nav(soup, len_section_array, is_copy=False, manifest=None):
    items = soup.find_all('nav')    
    idx = 0
    m_type = 'nav-bar'

    for item in items:        
        if item:                    
            idx += 1
            m_code = f'{len_section_array}-nav-bar-{idx}'        
            if manifest is not None:
                manifest.append(m_code) 
            # data-modal-title='{item.get_text(separator=', ', strip=True)}'            
            if not is_copy:
                replacement_soup = get_replacement_soup(item, m_code, m_type)            
            else:
                replacement_soup = get_replacement_soup_copy(item, m_code, m_type)
            item.replace_with(replacement_soup)

def scrape_text(filepath):
    '''
        Scrape all text from a local html file
        convert to hoverable text, and identifier (for ajax edit later)

        Note:
        Baca file asli html index.html
        Hasilnya di simpan di res.html dan di source.html
        1. Source.html untuk di load di ajax edit (tanpa hoverable)
        2. Res.html untuk di tampilkan di browser (ada hoverable)

        Cara pakai:
        1. Create file index.html dengan id yg sama seperti res.html
        2. Create file res.html (penghubungnya id)

        Karna belum konek ke database
        maka perlu buat file manifest.json untuk menyimpan daftar id       
        id ini digunakan oleh ajax edit untuk mengedit data atau menampilkan data          
    '''    
    # URL of the webpage to scrape
    # url = "./index.html"
    # Membuka file HTML lokal
    with open(filepath, "r", encoding="utf-8") as file:
        content = file.read()

    # remove all space, tab, new lines
    content = content.replace('\n', ' ').replace('\r', ' ').replace('\t', ' ').replace('$','Rp')

    # remove all multiple spaces with single space
    cleaned_text = re.sub(r'\s+', ' ', content).strip()
    
    # Membuat objek BeautifulSoup "lxml" "html5lib"
    soup = BeautifulSoup(cleaned_text, "html.parser")

    # Remove all script and style elements
    comments = soup.find_all(string=lambda text: isinstance(text, Comment))
    for comment in comments:
        comment.extract()

    # Create a deep copy of the soup object
    # Deep Copy: The copy.deepcopy() function ensures that the copied object is independent of the original.
    soup_copy = copy.deepcopy(soup)

    # Remove href from all <a> tags
    for a_tag in soup.find_all('a'):
        if 'href' in a_tag.attrs:
            del a_tag.attrs['href']    
    
    # Extract all visible text from the body tag
    # OKE FIND ALL TEXT
    # -----------------
    # get all text with new line separator
    body_text = soup.body.get_text(separator="\n", strip=True)    

    # split by new line
    tmp = body_text.split('\n')
    
    m_section_array = []    
    m_section_array.append({ 'name': 'main', 'code': 0 })

    m_section_array_copy = []    
    m_section_array_copy.append({ 'name': 'main', 'code': 0 })

    m_manifest = [] # untuk menyimpan daftar id yang akan di simpan di manifest.json    

    for i in range(len(tmp)):        
        add_hover_click(soup, tmp[i], m_section_array, manifest=m_manifest)
        add_hover_click(soup_copy, tmp[i], m_section_array_copy, is_copy=True)
    
    replace_icon(soup, 'bi', len(m_section_array), manifest=m_manifest) # bi is bootstrap icon
    replace_icon(soup_copy, 'bi', len(m_section_array_copy), is_copy=True) # bi is bootstrap icon

    replace_img(soup, len(m_section_array), manifest=m_manifest)
    replace_img(soup_copy, len(m_section_array_copy), is_copy=True)

    replace_ul(soup, len(m_section_array), manifest=m_manifest)
    replace_ul(soup_copy, len(m_section_array_copy), is_copy=True)

    replace_nav(soup, len(m_section_array), manifest=m_manifest)
    replace_nav(soup_copy, len(m_section_array_copy), is_copy=True)
        
    with open("res.html", "w", encoding="utf-8") as file:
        file.write(soup.prettify())

    with open("src.html", "w", encoding="utf-8") as file:
        file.write(soup_copy.prettify())

    # print('manifest', m_manifest)
    with open("manifest.json", "w", encoding="utf-8") as file:
        # file.write(m_manifest.json())       
        json.dump(m_manifest, file) 

    print("Scraping completed. Results saved to res.html, src.html, and manifest.json")
